from .pyxwb2 import XwingSquadron, XwingDataPack

__version__ = "0.2.0b"
__vendors__ = {
    "xws": "2.0",
    "xwing-data": "1.25.0",
    "dial-maneuvers-conversion": "1.0.0",
    "jsonschema": "2019-09"
}
